///////////////////////////////////////////////////////////////////////

// SVector.h

//

// Copyright by Andr Stein

// E-Mail: stone@steinsoft.net, andre_stein@web.de

// http://www.steinsoft.net

//////////////////////////////////////////////////////////////////////



#ifndef SVECTOR_H

#define SVECTOR_H



#include <math.h>



class SVector

{

public:

	//*****************

	//Constructors

	//*****************

	SVector()

	{

		point[0] = 0.0f; point[1] = 0.0f; point[2] = 0.0f;

	}



	SVector(float x,float y,float z = 0.0f)

	{

		point[0] = x; point[1] = y; point[2] = z;

	}



	//*****************

	//Functions

	//*****************



	const float getX() const { return point[0]; }

	const float getY() const { return point[1]; }

	const float getZ() const { return point[2]; }



	void setX(const float& x) { point[0] = x; }

	void setY(const float& y) { point[0] = y; }

	void setZ(const float& z) { point[0] = z; }



	const float getLength() const

	{

		return sqrtf(point[0]*point[0] + point[1]*point[1] +

				point[2]*point[2]);

	}



	const float getAngle(const SVector& rhs) const

	{

		return acosf(operator %(rhs));

	}



	const SVector& reflection(const SVector& rhs) const

	{

		const SVector& vector = getUnitVector();



		return ( vector - rhs * 2.0f * ( vector % rhs ) ) * 

			getLength() ;

	}



	void normalize()

	{

		operator /= (getLength());

	}



	const SVector& getUnitVector() const

	{

		return operator / (getLength());

	}



	void operator |= (const float length )

	{

		operator = ( operator | (length) );

	}



	



	//****************

	//Operators

	//****************



	//comparasion operators



	const bool operator == (const SVector& rhs) const

	{

		return (point[0] == rhs.point[0] && point[1] == rhs.point[1]

			&& point[2] == rhs.point[2]);

	}



	const bool operator  != (const SVector& rhs) const

	{

		return (point[0] != rhs.point[0] && point[1] != rhs.point[1]

			&& point[2] != rhs.point[2]);

	}



	//type cast operators



	operator float*() const

	{

		return (float*)point;

	}



	//[] operator

	

	float& operator [] (const int index)

	{

		if (index >= 0 && index <= 2)

			return point[index];

	}



	//binary operators



	SVector operator + (const SVector& vector) const

	{

		return SVector(point[0] + vector.point[0],

			point[1] + vector.point[1],

			point[2] + vector.point[2]);

	}



	SVector operator * (const SVector& vector) const

	{

		return SVector(point[0] * vector.point[0],

			point[1] * vector.point[1],

			point[2] * vector.point[2]);

	}



	//v * k (scalar)

	SVector operator * (const float scalar) const

	{

		return SVector(point[0] * scalar,

			point[1] * scalar,

			point[2] * scalar);

	}



	SVector operator / (const SVector& vector) const

	{

		return SVector(point[0] / vector.point[0],

			point[1] / vector.point[1],

			point[2] / vector.point[2]);

	}



	//v / k (scalar)

	SVector operator / (const float scalar) const

	{

		return SVector(point[0] / scalar,

			point[1] / scalar,

			point[2] / scalar);

	}



	SVector operator - (const SVector& vector) const

	{

		return SVector(point[0] - vector.point[0],

			point[1] - vector.point[1],

			point[2] - vector.point[2]);

	}



	//dot product

	const float operator % (const SVector& rhs) const

	{

		return point[0]*rhs.point[0] + point[1]*rhs.point[1] +

			point[2]*rhs.point[2];

	}



	//cross product

	const SVector& operator ^ (const SVector& rhs) const

	{

		return SVector(point[1]*rhs.point[2] - point[2]*rhs.point[1],

			point[2]*rhs.point[0] - point[0]*rhs.point[2],

			point[0]*rhs.point[1] - point[1]*rhs.point[0]);

	}



	//construct vector with 'length' length

	const SVector& operator | (const float length) const

	{

		return operator * ( length / getLength() );

	}



	//+-*/= operators



	void operator += (const SVector& vector)

	{

		point[0] += vector.point[0]; 

		point[1] += vector.point[1];

		point[2] += vector.point[2];

	}



	void operator -= (const SVector& vector)

	{

		point[0] -= vector.point[0]; 

		point[1] -= vector.point[1];

		point[2] -= vector.point[2];

	}



	void operator *= (const SVector& vector)

	{

		point[0] *= vector.point[0]; 

		point[1] *= vector.point[1];

		point[2] *= vector.point[2];

	}



	void operator *= (const float scalar)

	{

		point[0] *= scalar; 

		point[1] *= scalar;

		point[2] *= scalar;

	}



	void operator /= (const SVector& vector)

	{

		point[0] /= vector.point[0]; 

		point[1] /= vector.point[1];

		point[2] /= vector.point[2];

	}



	void operator /= (const float scalar)

	{

		point[0] /= scalar; 

		point[1] /= scalar;

		point[2] /= scalar;

	}



	//unary operators



	SVector operator - () const

	{

		return SVector(-point[0],-point[1],-point[2]);

	}



protected:

	float point[3];

};



#endif 